import { NextRequest, NextResponse } from 'next/server';
import Link from 'next/link';

/**
 * Middleware function to handle proxy requests
 * @param request - The incoming Next.js request
 * @returns A modified NextResponse for proxy routes, or undefined for non-proxy routes
 *
 * @example
 * ```typescript
 * import { proxyMiddleware } from './middleware/proxy';
 *
 * export default function middleware(request: NextRequest) {
 *   return proxyMiddleware(request);
 * }
 * ```
 */
declare function proxyMiddleware(request: NextRequest): NextResponse<unknown> | undefined;

/**
 * Utility type for autocomplete suggestions
 * @template U - The type to be autocompleted, must extend T
 * @template T - The base type, defaults to string
 */
type Autocomplete<U extends T, T = string> = U | (T & Record<never, never>);
/**
 * Utility type to add a wildcard to a path pattern
 */
type WithPathPatternWildcard<T> = `${T & string}(.*)`;
/**
 * Extracts the string type from Link href prop
 */
type NextTypedRoute<T = Parameters<typeof Link>["0"]["href"]> = T extends string ? T : never;
/**
 * Type for route matcher that supports Next.js typed routes
 */
type RouteMatcherWithNextTypedRoutes = Autocomplete<WithPathPatternWildcard<NextTypedRoute> | NextTypedRoute>;
/**
 * Union type for possible route matcher parameters
 */
type RouteMatcherParam = Array<RegExp | RouteMatcherWithNextTypedRoutes> | RegExp | RouteMatcherWithNextTypedRoutes | ((req: NextRequest) => boolean);
/**
 * Creates a route matcher function based on the provided routes
 * @param routes - The routes to match against
 * @returns A function that accepts a NextRequest and returns a boolean indicating if the request matches any of the routes
 * @throws {Error} If the path pattern is invalid
 *
 * @example
 * ```typescript
 * const matcher = createRouteMatcher(['/foo', '/bar(.*)']);
 * const isMatch = matcher(request);
 * ```
 */
declare const createRouteMatcher: (routes: RouteMatcherParam) => (req: NextRequest) => boolean;

export { createRouteMatcher, proxyMiddleware };
